"use import";

import lib.Callback;
import lib.PubSub;

from net.protocols.rtjp import RTJPProtocol;

var Error = Class(function() {
	this.init = function(protocol, id, msg, details, requestId) {
		this.id = id;
		this.msg = msg;
		this.details = details;
		this.requestId = requestId;
	}
});

var RPCRequest = Class(function() {
	this.init = function(protocol, id) {
		this.protocol = protocol;
		this.id = id;
		this._onError = new lib.Callback();
		this._onSuccess = new lib.Callback();
	}

	this.onError = function() { this._onError.forward(arguments); }
	this.onSuccess = function() { this._onSuccess.forward(arguments); }
	
	this.bindLater = function(l) {
		var args = [].slice(arguments, 1);
		this._onError.forward([l, l.fail].concat(args));
		this._onSuccess.forward([l, l.succed].concat(args));
		return l;
	}
	
	
});

var ReceivedRequest = Class(function() {
    this.type = "request"

   this.init = function(protocol, id, name, args, target) {
        this.protocol = protocol
        this.id = id;
        this.name = name
        this.responded = false;
        this.args = args;
		this.target = target;
    }

    this.error = function(msg, details) {
        if (this.responded) { throw new Error("already responded"); }
        args = {
            id: this.id,
            msg: msg + ""
        }
        if (details !== undefined) { args.details = details }
        this.responded = true;        
        this.protocol.sendFrame('ERROR', args);
    }

    this.respond = function(args) {
        if (this.responded) { throw new Error("already responded"); }
        this.responded = true;
        this.protocol.sendFrame('RESPONSE', {
            id: this.id,
            args: args
        });
    }
});

var ReceivedEvent = Class(function() {
	this.init = function(protocol, id, name, args, target) {
		this.id = id;
		this.name = name;
		this.args = args;
		this.target = target;
	}
});

exports.CuppaProtocol = Class(RTJPProtocol, function(supr) {
	this.init = function() {
		supr(this, 'init', arguments);
		
		this._onConnect = new lib.Callback();
		this._onDisconnect = new lib.Callback();
		
		this._connected = false;
		this._requests = {};
		
		this.onEvent = new lib.PubSub();
		this.onRequest = new lib.PubSub();
	}
	
	// pass something to call (ctx, method, args...) when connected
	this.onConnect = function() { this._onConnect.forward(arguments); }
	this.onDisconnect = function() { this._onDisconnect.forward(arguments); }
	
	// called when we're connected
	this.connectionMade = function() {
		this._connected = true;
		this._onConnect.fire();
	}
	
	this.connectionLost = function(err) {
		this._connected = false;
		this._onDisconnect.fire(err);
	}
	
	this.sendRequest = function(name, args, cb, target) {
		if (typeof(cb) != 'function') {
			target = cb;
			cb = null;
		}
		var id = this.sendFrame('RPC', {name: name, args: args, target: target || null}),
			req = this._requests[id] = new RPCRequest(this, id);
		if (cb) {
			req.onSuccess(bind(GLOBAL, cb, true));
			req.onError(bind(GLOBAL, cb, false));
		}
		return req;
	}
	
	this.sendEvent = function(name, args, target) {
		this.sendFrame('EVENT', {name: name, args: args, target: target || null});
	}
	
	this.frameReceived = function(id, name, args) {
		logger.debug('RECEIVED', id, name, args);
		switch(name.toUpperCase()) {
			case 'RESPONSE':
				var req = this._requests[args.id];
				if (!req) { return; }
				delete this._requests[requestId];
				req._onSuccess.fire(args.args);
				break;
			case 'ERROR':
				var msg = args.msg || 'unknown',
					requestId = args.id,
					req = this._requests[requestId],
					err = new Error(this, id, msg, args.details, requestId);
				
				if (!req) {
					return this.errorReceived(err);
				} else {
					delete this._requests[requestId];
					req._onError.fire(err);
				}
				break;
			case 'RPC':
			case 'EVENT':
				if (!args.name) {
					return self.sendFrame('ERROR', { 'id': id, 'msg': 'missing "name"' });
				}
				
				var frameArgs = args.args || {},
					target = args.target || null,
					isRPC = name.toUpperCase() == 'RPC',
					reqCtor = isRPC ? ReceivedRequest : ReceivedEvent,
					pubTarget = isRPC ? this.onRequest : this.onEvent,
					req = new reqCtor(this, id, args.name, frameArgs, target);
				
				pubTarget.publish(req.name, req);
				break;
			default:
				break;
		}
	}
});
